@@ -20,6 +20,9 @@
 #define FLICKR_PHOTO_ORIG_URL                           \
   "http://farm%s.static.flickr.com/%s/%s_%s_o.%s"
 
+#define FLICKR_PHOTO_SMALL_URL                          \
+  "http://farm%s.static.flickr.com/%s/%s_%s_n.jpg"
+
 #define FLICKR_PHOTO_THUMB_URL                          \
   "http://farm%s.static.flickr.com/%s/%s_%s_t.jpg"
 
@@ -637,6 +640,34 @@ g_flickr_photo_url_original (GFlickr *f, GHashTable *photo)
   }
 }
 
+gchar *
+g_flickr_photo_url_small (GFlickr *f, GHashTable *photo)
+{
+  gchar *farm_id;
+  gchar *secret;
+  gchar *photo_id;
+  gchar *server_id;
+
+  if (!photo) {
+    return NULL;
+  }
+
+  farm_id = g_hash_table_lookup (photo, "photo_farm");
+  secret = g_hash_table_lookup (photo, "photo_secret");
+  photo_id = g_hash_table_lookup (photo, "photo_id");
+  server_id = g_hash_table_lookup (photo, "photo_server");
+
+  if (!farm_id || !secret || !photo_id || !server_id) {
+    return NULL;
+  } else {
+    return g_strdup_printf (FLICKR_PHOTO_SMALL_URL,
+                            farm_id,
+                            server_id,
+                            photo_id,
+                            secret);
+  }
+}
+
 gchar *
 g_flickr_photo_url_thumbnail (GFlickr *f, GHashTable *photo)
 {
