@@ -26,6 +26,7 @@
 
 #include <glib.h>
 #include <glib/gstdio.h>
+#include <glib/gi18n-lib.h>
 #include <grilo.h>
 #include <sqlite3.h>
 #include <string.h>
@@ -43,8 +44,8 @@ GRL_LOG_DOMAIN_STATIC(metadata_store_log_domain);
 #define PLUGIN_ID   METADATA_STORE_PLUGIN_ID
 
 #define SOURCE_ID   "grl-metadata-store"
-#define SOURCE_NAME "Metadata Store"
-#define SOURCE_DESC "A plugin for storing extra metadata information"
+#define SOURCE_NAME _("Metadata Store")
+#define SOURCE_DESC _("A plugin for storing extra metadata information")
 
 #define GRL_SQL_DB  "grl-metadata-store.db"
 
@@ -91,6 +92,9 @@ GRL_LOG_DOMAIN_STATIC(metadata_store_log_domain);
 #define GRL_SQL_SOURCE_FILTER                   \
   "source_id=?"
 
+#define GRL_SQL_TYPE_FILTER                     \
+  "type_id IN ( ? , ? , ? )"
+
 #define GRL_SQL_SEARCH_FILTER                   \
   "SELECT * FROM store "                        \
   "WHERE %s "                                   \
@@ -158,6 +162,10 @@ grl_metadata_store_source_plugin_init (GrlRegistry *registry,
 
   GRL_DEBUG ("grl_metadata_store_source_plugin_init");
 
+  /* Initialize i18n */
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+
   GrlMetadataStoreSource *source = grl_metadata_store_source_new ();
   grl_registry_register_source (registry,
                                 plugin,
@@ -563,8 +571,8 @@ write_keys (sqlite3 *db,
                      "keys is writable");
     *error = g_error_new (GRL_CORE_ERROR,
                           GRL_CORE_ERROR_STORE_METADATA_FAILED,
-                          "Failed to update metadata, "
-                          "specified keys are not writable");
+                          _("Failed to update metadata: %s"),
+                          _("specified keys are not writable"));
     goto done;
   }
 
@@ -582,7 +590,8 @@ write_keys (sqlite3 *db,
     failed_keys = g_list_copy (sms->keys);
     *error = g_error_new (GRL_CORE_ERROR,
                           GRL_CORE_ERROR_STORE_METADATA_FAILED,
-                          "Failed to update metadata");
+                          _("Failed to update metadata: %s"),
+                          sqlite3_errmsg (db));
     goto done;
   }
 
@@ -601,9 +610,9 @@ write_keys (sqlite3 *db,
                      source_id, media_id, sqlite3_errmsg (db));
     g_list_free (failed_keys);
     failed_keys = g_list_copy (sms->keys);
-    *error = g_error_new (GRL_CORE_ERROR,
-                          GRL_CORE_ERROR_STORE_METADATA_FAILED,
-                          "Failed to update metadata");
+    *error = g_error_new_literal (GRL_CORE_ERROR,
+                                  GRL_CORE_ERROR_STORE_METADATA_FAILED,
+                                  _("Failed to update metadata"));
     goto done;
   }
 
@@ -691,6 +700,7 @@ grl_metadata_store_source_get_caps (GrlSource *source,
                                         GRL_METADATA_KEY_INVALID);
       grl_caps_set_key_filter (caps, keys);
       g_list_free (keys);
+      grl_caps_set_type_filter (caps, GRL_TYPE_FILTER_ALL);
   }
 
   return caps;
@@ -746,8 +756,9 @@ grl_metadata_store_source_resolve (GrlSource *source,
   if (!source_id) {
     GRL_WARNING ("Failed to resolve metadata: source-id not available");
     error = g_error_new (GRL_CORE_ERROR,
-			 GRL_CORE_ERROR_RESOLVE_FAILED,
-			 "source-id not available, cannot resolve metadata.");
+                         GRL_CORE_ERROR_RESOLVE_FAILED,
+                         _("Failed to resolve: %s"),
+                         _("\"source-id\" not available"));
     rs->callback (rs->source, rs->operation_id, rs->media, rs->user_data, error);
     g_error_free (error);
     return;
@@ -765,9 +776,9 @@ grl_metadata_store_source_resolve (GrlSource *source,
     rs->callback (rs->source, rs->operation_id, rs->media, rs->user_data, NULL);
   } else {
     GRL_WARNING ("Failed to resolve metadata");
-    error = g_error_new (GRL_CORE_ERROR,
-			 GRL_CORE_ERROR_RESOLVE_FAILED,
-			 "Failed to resolve metadata.");
+    error = g_error_new_literal (GRL_CORE_ERROR,
+                                 GRL_CORE_ERROR_RESOLVE_FAILED,
+                                 _("Failed to resolve"));
     rs->callback (rs->source, rs->operation_id, rs->media, rs->user_data, error);
     g_error_free (error);
   }
@@ -791,7 +802,8 @@ grl_metadata_store_source_store_metadata (GrlSource *source,
     GRL_WARNING ("Failed to update metadata: source-id not available");
     error = g_error_new (GRL_CORE_ERROR,
                          GRL_CORE_ERROR_STORE_METADATA_FAILED,
-                         "source-id not available, cannot update metadata.");
+                         _("Failed to update metadata: %s"),
+                         _("\"source-id\" not available"));
     failed_keys = g_list_copy (sms->keys);
   } else {
     /* Special case for root categories */
@@ -819,22 +831,25 @@ grl_metadata_store_source_search (GrlSource *source,
   sqlite3 *db;
   gchar *sql;
   gint r;
+  gint i;
   GError *error = NULL;
   GrlMedia *media;
   GList *iter, *medias = NULL;
   GValue *filter_favourite_val;
   GValue *filter_source_val;
+  GrlTypeFilter filter_type_val;
   GString *filters;
   guint count;
+  gint type_filter[3];
 
   GRL_DEBUG (__FUNCTION__);
 
   db = GRL_METADATA_STORE_SOURCE (source)->priv->db;
   if (!db) {
     GRL_WARNING ("Can't execute operation: no database connection.");
-    error = g_error_new (GRL_CORE_ERROR,
-                         GRL_CORE_ERROR_QUERY_FAILED,
-                         "No database connection");
+    error = g_error_new_literal (GRL_CORE_ERROR,
+                                 GRL_CORE_ERROR_QUERY_FAILED,
+                                 _("No database connection"));
     ss->callback (ss->source, ss->operation_id, NULL, 0, ss->user_data, error);
     g_error_free (error);
     return;
@@ -846,6 +861,8 @@ grl_metadata_store_source_search (GrlSource *source,
                                                                GRL_METADATA_KEY_FAVOURITE);
   filter_source_val = grl_operation_options_get_key_filter (ss->options,
                                                             GRL_METADATA_KEY_SOURCE);
+  filter_type_val = grl_operation_options_get_type_filter (ss->options);
+
   if (filter_favourite_val) {
     filters = g_string_append (filters, GRL_SQL_FAVOURITE_FILTER);
   }
@@ -857,6 +874,29 @@ grl_metadata_store_source_search (GrlSource *source,
     filters = g_string_append (filters, GRL_SQL_SOURCE_FILTER);
   }
 
+  if (filter_type_val != GRL_TYPE_FILTER_ALL) {
+    /* Fill the type_filter array */
+    if (filter_type_val & GRL_TYPE_FILTER_AUDIO) {
+      type_filter[0] = MEDIA_AUDIO;
+    } else {
+      type_filter[0] = -1;
+    }
+    if (filter_type_val & GRL_TYPE_FILTER_VIDEO) {
+      type_filter[1] = MEDIA_VIDEO;
+    } else {
+      type_filter[1] = -1;
+    }
+    if (filter_type_val & GRL_TYPE_FILTER_IMAGE) {
+      type_filter[2] = MEDIA_IMAGE;
+    } else {
+      type_filter[2] = -1;
+    }
+    if (filters->len > 0) {
+      filters = g_string_append (filters, " AND ");
+    }
+    filters = g_string_append (filters, GRL_SQL_TYPE_FILTER);
+  }
+
   if (filters->len > 0) {
     sql = g_strdup_printf (GRL_SQL_SEARCH_FILTER,
                            filters->str,
@@ -877,7 +917,8 @@ grl_metadata_store_source_search (GrlSource *source,
     GRL_WARNING ("Failed to search in the metadata store: %s", sqlite3_errmsg (db));
     error = g_error_new (GRL_CORE_ERROR,
                          GRL_CORE_ERROR_SEARCH_FAILED,
-                         "Failed to search in the metadata store");
+                         _("Failed to search: %s"),
+                         sqlite3_errmsg (db));
     ss->callback (ss->source, ss->operation_id, NULL, 0, ss->user_data, error);
     g_error_free (error);
     return;
@@ -893,6 +934,12 @@ grl_metadata_store_source_search (GrlSource *source,
     sqlite3_bind_text (sql_stmt, count++, g_value_get_string (filter_source_val), -1, SQLITE_STATIC);
   }
 
+  if (filter_type_val != GRL_TYPE_FILTER_ALL) {
+    for (i = 0; i < G_N_ELEMENTS (type_filter); i++) {
+      sqlite3_bind_int (sql_stmt, count++, type_filter[i]);
+    }
+  }
+
   while ((r = sqlite3_step (sql_stmt)) == SQLITE_BUSY);
 
   count = 0;
@@ -907,7 +954,8 @@ grl_metadata_store_source_search (GrlSource *source,
     GRL_WARNING ("Failed to search in the metadata store: %s", sqlite3_errmsg (db));
     error = g_error_new (GRL_CORE_ERROR,
                          GRL_CORE_ERROR_SEARCH_FAILED,
-                         "Failed to search in the metadata store");
+                         _("Failed to search: %s"),
+                         sqlite3_errmsg (db));
     ss->callback (ss->source, ss->operation_id, NULL, 0, ss->user_data, error);
     g_error_free (error);
     sqlite3_finalize (sql_stmt);
