@@ -26,6 +26,7 @@
 
 #include <grilo.h>
 #include <gio/gio.h>
+#include <glib/gi18n-lib.h>
 #include <totem-pl-parser.h>
 #include <string.h>
 #include <stdlib.h>
@@ -42,8 +43,8 @@ GRL_LOG_DOMAIN_STATIC(optical_media_log_domain);
 #define PLUGIN_ID   OPTICAL_MEDIA_PLUGIN_ID
 
 #define SOURCE_ID   "grl-optical-media"
-#define SOURCE_NAME "Optical Media"
-#define SOURCE_DESC "A source for browsing optical media"
+#define SOURCE_NAME _("Optical Media")
+#define SOURCE_DESC _("A source for browsing optical media")
 
 /* --- Grilo OpticalMedia Private --- */
 
@@ -93,6 +94,10 @@ grl_optical_media_plugin_init (GrlRegistry *registry,
 
   GRL_DEBUG ("%s", __FUNCTION__);
 
+  /* Initialize i18n */
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+
   GrlOpticalMediaSource *source = grl_optical_media_source_new ();
 
   grl_registry_register_source (registry,
@@ -235,56 +240,6 @@ get_uri_for_gicon (GIcon *icon)
   return uri;
 }
 
-static GList *
-add_mount (GList *media_list,
-           GMount *mount,
-           GrlOpticalMediaSource *source)
-{
-  char *name, *icon_uri;
-  GIcon *icon;
-  GrlMedia *media;
-  char *id;
-
-  GVolume *volume;
-  GFile *root;
-
-  /* Check whether we have an archive mount */
-  volume = g_mount_get_volume (mount);
-  if (volume != NULL) {
-    g_object_unref (volume);
-    return media_list;
-  }
-
-  root = g_mount_get_root (mount);
-  if (g_file_has_uri_scheme (root, "archive") == FALSE) {
-    g_object_unref (root);
-    return media_list;
-  }
-
-  media = grl_media_video_new ();
-
-  id = g_file_get_uri (root);
-  grl_media_set_id (media, id);
-  g_free (id);
-
-  /* Work out an icon to display */
-  icon = g_mount_get_icon (mount);
-  icon_uri = get_uri_for_gicon (icon);
-  g_object_unref (icon);
-  grl_media_set_thumbnail (media, icon_uri);
-  g_free (icon_uri);
-
-  /* Get the mount's pretty name for the menu label */
-  name = g_mount_get_name (mount);
-  g_strstrip (name);
-  grl_media_set_title (media, name);
-  g_free (name);
-
-  grl_media_set_mime (media, "x-special/device-block");
-
-  return g_list_prepend (media_list, media);
-}
-
 static GList *
 add_volume (GList *media_list,
             GVolume *volume,
@@ -350,12 +305,26 @@ add_drive (GList *media_list,
            GrlOpticalMediaSource *source)
 {
   GList *volumes, *i;
+  GIcon *icon;
 
   if (g_drive_can_eject (drive) == FALSE ||
       g_drive_has_media (drive) == FALSE) {
     return media_list;
   }
 
+  /* Hack to avoid USB devices showing up
+   * https://bugzilla.gnome.org/show_bug.cgi?id=679624 */
+  icon = g_drive_get_icon (drive);
+  if (icon && G_IS_THEMED_ICON (icon)) {
+    const gchar * const * names;
+    names = g_themed_icon_get_names (G_THEMED_ICON (icon));
+    if (names && names[0] && !g_str_has_prefix (names[0], "drive-optical")) {
+      g_object_unref (icon);
+      return media_list;
+    }
+  }
+  g_clear_object (&icon);
+
   /* Repeat for all the drive's volumes */
   volumes = g_drive_get_volumes (drive);
 
@@ -476,7 +445,7 @@ grl_optical_media_source_browse (GrlSource *source,
                                  GrlSourceBrowseSpec *bs)
 {
   GList *drives;
-  GList *mounts;
+  GList *volumes;
   GList *l;
   GrlOpticalMediaSourcePrivate *priv = GRL_OPTICAL_MEDIA_SOURCE (source)->priv;
   BrowseData *data;
@@ -496,15 +465,21 @@ grl_optical_media_source_browse (GrlSource *source,
   }
   g_list_free (drives);
 
-  /* Look for mounted archives */
-  mounts = g_volume_monitor_get_mounts (priv->monitor);
-  for (l = mounts; l != NULL; l = l->next) {
-    GMount *mount = l->data;
+  /* Look for mounted ISO images */
+  volumes = g_volume_monitor_get_volumes (priv->monitor);
+  for (l = volumes; l != NULL; l = l->next) {
+    GVolume *volume = l->data;
+    char *path;
 
-    media_list = add_mount (media_list, mount, GRL_OPTICAL_MEDIA_SOURCE (source));
-    g_object_unref (mount);
+    path = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+
+    if (path != NULL && g_str_has_prefix (path, "/dev/loop"))
+      media_list = add_volume (media_list, volume, NULL, GRL_OPTICAL_MEDIA_SOURCE (source));
+
+    g_free (path);
+    g_object_unref (volume);
   }
-  g_list_free (mounts);
+  g_list_free (volumes);
 
   /* Got nothing? */
   if (media_list == NULL) {
@@ -530,6 +505,7 @@ grl_optical_media_source_browse (GrlSource *source,
   grl_operation_set_data (bs->operation_id, data->cancellable);
 
   data->parser = totem_pl_parser_new ();
+  g_object_set (data->parser, "recurse", FALSE, NULL);
   g_signal_connect (G_OBJECT (data->parser), "entry-parsed",
                     G_CALLBACK (entry_parsed_cb), data);
 
