@@ -521,7 +521,14 @@ device_unavailable_cb (GUPnPControlPoint *cp,
   /* Check first if source is registered */
   source = grl_registry_lookup_source (registry, source_id);
   if (source) {
+    GError *error = NULL;
     GRL_DEBUG ("Unregistered source %s", source_id);
+    grl_registry_unregister_source (registry, source, &error);
+    if (error)
+      {
+        g_warning ("Failed to unregister source %s: %s", source_id, error->message);
+        g_error_free (error);
+      }
     g_free (source_id);
     return;
   }
@@ -1118,6 +1125,7 @@ gupnp_resolve_cb (GUPnPServiceProxy *service,
 {
   GError *error = NULL;
   gchar *didl = NULL;
+  guint returned = 0;
   GrlSourceResolveSpec *rs;
   GUPnPDIDLLiteParser *didl_parser;
 
@@ -1128,11 +1136,12 @@ gupnp_resolve_cb (GUPnPServiceProxy *service,
 
   gupnp_service_proxy_end_action (service, action, &error,
                                   "Result", G_TYPE_STRING, &didl,
+                                  "NumberReturned", G_TYPE_UINT, &returned,
                                   NULL);
 
-  if (!didl) {
-    GRL_DEBUG ("Resolve operation failed");
-    rs->callback (rs->source, rs->operation_id, rs->media, rs->user_data, error);
+  if (!didl || !returned) {
+    GRL_DEBUG ("Got no results for resolve");
+    rs->callback (rs->source, rs->operation_id, rs->media, rs->user_data, error? error: NULL);
     if (error) {
       g_error_free (error);
     }
@@ -1228,7 +1237,7 @@ grl_upnp_source_browse (GrlSource *source,
     action =
       gupnp_service_proxy_begin_action (GRL_UPNP_SOURCE (source)->priv->service,
                                         "Search", gupnp_browse_cb, os,
-                                        "ContainerID", G_TYPE_STRING, container_id,
+                                        "ContainerID", G_TYPE_STRING, "0",
                                         "SearchCriteria", G_TYPE_STRING, upnp_browse,
                                         "Filter", G_TYPE_STRING, upnp_filter,
                                         "StartingIndex", G_TYPE_UINT, os->skip,
